@@ -260,6 +260,44 @@ g_mime_gpgme_sign (gpgme_ctx_t ctx, gpgme_sig_mode_t mode, const char *userid,
 	return (GMimeDigestAlgo) result->signatures->hash_algo;
 }
 
+
+/* return TRUE iff a < b */
+static gboolean
+_gpgv_lt(gpgme_validity_t a, gpgme_validity_t b) {
+	switch (a) {
+	case GPGME_VALIDITY_NEVER:
+		return b != GPGME_VALIDITY_NEVER;
+	case GPGME_VALIDITY_UNKNOWN:
+	case GPGME_VALIDITY_UNDEFINED:
+		switch (b) {
+		case GPGME_VALIDITY_NEVER:
+		case GPGME_VALIDITY_UNKNOWN:
+		case GPGME_VALIDITY_UNDEFINED:
+			return FALSE;
+		default:
+			return TRUE;
+		}
+	case GPGME_VALIDITY_MARGINAL:
+		switch (b) {
+		case GPGME_VALIDITY_NEVER:
+		case GPGME_VALIDITY_UNKNOWN:
+		case GPGME_VALIDITY_UNDEFINED:
+		case GPGME_VALIDITY_MARGINAL:
+			return FALSE;
+		default:
+			return TRUE;
+		}
+	case GPGME_VALIDITY_FULL:
+		return b == GPGME_VALIDITY_ULTIMATE;
+	case GPGME_VALIDITY_ULTIMATE:
+		return FALSE;
+	default:
+		g_assert_not_reached();
+		return FALSE;
+	}
+}
+
+
 static GMimeSignatureList *
 g_mime_gpgme_get_signatures (gpgme_ctx_t ctx, gboolean verify)
 {
@@ -290,6 +328,7 @@ g_mime_gpgme_get_signatures (gpgme_ctx_t ctx, gboolean verify)
 		g_mime_certificate_set_pubkey_algo (signature->cert, (GMimePubKeyAlgo) sig->pubkey_algo);
 		g_mime_certificate_set_digest_algo (signature->cert, (GMimeDigestAlgo) sig->hash_algo);
 		g_mime_certificate_set_fingerprint (signature->cert, sig->fpr);
+		g_mime_certificate_set_key_id (signature->cert, sig->fpr);
 		
 		if (gpgme_get_key (ctx, sig->fpr, &key, 0) == GPG_ERR_NO_ERROR && key) {
 			/* get more signer info from their signing key */
@@ -297,23 +336,37 @@ g_mime_gpgme_get_signatures (gpgme_ctx_t ctx, gboolean verify)
 			g_mime_certificate_set_issuer_serial (signature->cert, key->issuer_serial);
 			g_mime_certificate_set_issuer_name (signature->cert, key->issuer_name);
 			
-			/* get the keyid, name, and email address */
+			gpgme_validity_t validity = GPGME_VALIDITY_NEVER;
+			gboolean founduid = FALSE;
+ 			
+			/* get the most valid name, email address, and full user id */
 			uid = key->uids;
 			while (uid) {
-				if (uid->name && *uid->name)
-					g_mime_certificate_set_name (signature->cert, uid->name);
-				
-				if (uid->email && *uid->email)
-					g_mime_certificate_set_email (signature->cert, uid->email);
-				
-				if (uid->uid && *uid->uid)
-					g_mime_certificate_set_key_id (signature->cert, uid->uid);
-				
-				if (signature->cert->name && signature->cert->email && signature->cert->keyid)
-					break;
-				
+				if (!founduid || !_gpgv_lt (uid->validity, validity)) {
+					/* this is as good as the best UID we've found so far */
+					founduid = TRUE;
+					if (_gpgv_lt (validity, uid->validity)) {
+						/* this is actually better than the last best,
+						   so clear all the previously-found uids */
+						g_mime_certificate_set_name (signature->cert, NULL);
+						g_mime_certificate_set_email (signature->cert, NULL);
+						g_mime_certificate_set_user_id (signature->cert, NULL);
+					}
+					validity = uid->validity;
+
+					if (uid->name && *uid->name && !g_mime_certificate_get_name (signature->cert))
+						g_mime_certificate_set_name (signature->cert, uid->name);
+					
+					if (uid->email && *uid->email && !g_mime_certificate_get_email (signature->cert))
+						g_mime_certificate_set_email (signature->cert, uid->email);
+					
+					if (uid->uid && *uid->uid && !g_mime_certificate_get_user_id (signature->cert))
+						g_mime_certificate_set_user_id (signature->cert, uid->uid);
+					
+				}
 				uid = uid->next;
 			}
+			g_mime_certificate_set_id_validity (signature->cert, (GMimeValidity)(validity));
 			
 			/* get the subkey used for signing */
 			subkey = key->subkeys;
