@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2014 Jeffrey Stedfast
+ *  Copyright (C) 2000-2017 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -25,6 +25,8 @@
 #include <glib.h>
 #include <glib-object.h>
 
+#include <gmime/gmime-format-options.h>
+#include <gmime/gmime-parser-options.h>
 #include <gmime/gmime-content-type.h>
 #include <gmime/gmime-disposition.h>
 #include <gmime/gmime-encodings.h>
@@ -59,24 +61,26 @@ struct _GMimeObject {
 	GMimeContentDisposition *disposition;
 	GMimeContentType *content_type;
 	GMimeHeaderList *headers;
-	
 	char *content_id;
+	
+	/* < private > */
+	gboolean ensure_newline;
 };
 
 struct _GMimeObjectClass {
 	GObjectClass parent_class;
 	
-	void         (* prepend_header) (GMimeObject *object, const char *header, const char *value);
-	void         (* append_header)  (GMimeObject *object, const char *header, const char *value);
-	void         (* set_header)     (GMimeObject *object, const char *header, const char *value);
-	const char * (* get_header)     (GMimeObject *object, const char *header);
-	gboolean     (* remove_header)  (GMimeObject *object, const char *header);
+	void         (* header_added)    (GMimeObject *object, GMimeHeader *header);
+	void         (* header_changed)  (GMimeObject *object, GMimeHeader *header);
+	void         (* header_removed)  (GMimeObject *object, GMimeHeader *header);
+	void         (* headers_cleared) (GMimeObject *object);
 	
 	void         (* set_content_type) (GMimeObject *object, GMimeContentType *content_type);
 	
-	char *       (* get_headers)   (GMimeObject *object);
+	char *       (* get_headers)   (GMimeObject *object, GMimeFormatOptions *options);
 	
-	ssize_t      (* write_to_stream) (GMimeObject *object, GMimeStream *stream);
+	ssize_t      (* write_to_stream) (GMimeObject *object, GMimeFormatOptions *options,
+					  gboolean content_only, GMimeStream *stream);
 	
 	void         (* encode) (GMimeObject *object, GMimeEncodingConstraint constraint);
 };
@@ -98,8 +102,8 @@ GType g_mime_object_get_type (void);
 
 void g_mime_object_register_type (const char *type, const char *subtype, GType object_type);
 
-GMimeObject *g_mime_object_new (GMimeContentType *content_type);
-GMimeObject *g_mime_object_new_type (const char *type, const char *subtype);
+GMimeObject *g_mime_object_new (GMimeParserOptions *options, GMimeContentType *content_type);
+GMimeObject *g_mime_object_new_type (GMimeParserOptions *options, const char *type, const char *subtype);
 
 void g_mime_object_set_content_type (GMimeObject *object, GMimeContentType *content_type);
 GMimeContentType *g_mime_object_get_content_type (GMimeObject *object);
@@ -112,24 +116,24 @@ GMimeContentDisposition *g_mime_object_get_content_disposition (GMimeObject *obj
 void g_mime_object_set_disposition (GMimeObject *object, const char *disposition);
 const char *g_mime_object_get_disposition (GMimeObject *object);
 
-void g_mime_object_set_content_disposition_parameter (GMimeObject *object, const char *attribute, const char *value);
-const char *g_mime_object_get_content_disposition_parameter (GMimeObject *object, const char *attribute);
+void g_mime_object_set_content_disposition_parameter (GMimeObject *object, const char *name, const char *value);
+const char *g_mime_object_get_content_disposition_parameter (GMimeObject *object, const char *name);
 
 void g_mime_object_set_content_id (GMimeObject *object, const char *content_id);
 const char *g_mime_object_get_content_id (GMimeObject *object);
 
-void g_mime_object_prepend_header (GMimeObject *object, const char *header, const char *value);
-void g_mime_object_append_header (GMimeObject *object, const char *header, const char *value);
-void g_mime_object_set_header (GMimeObject *object, const char *header, const char *value);
+void g_mime_object_prepend_header (GMimeObject *object, const char *header, const char *value, const char *charset);
+void g_mime_object_append_header (GMimeObject *object, const char *header, const char *value, const char *charset);
+void g_mime_object_set_header (GMimeObject *object, const char *header, const char *value, const char *charset);
 const char *g_mime_object_get_header (GMimeObject *object, const char *header);
 gboolean g_mime_object_remove_header (GMimeObject *object, const char *header);
 
 GMimeHeaderList *g_mime_object_get_header_list (GMimeObject *object);
 
-char *g_mime_object_get_headers (GMimeObject *object);
+char *g_mime_object_get_headers (GMimeObject *object, GMimeFormatOptions *options);
 
-ssize_t g_mime_object_write_to_stream (GMimeObject *object, GMimeStream *stream);
-char *g_mime_object_to_string (GMimeObject *object);
+ssize_t g_mime_object_write_to_stream (GMimeObject *object, GMimeFormatOptions *options, GMimeStream *stream);
+char *g_mime_object_to_string (GMimeObject *object, GMimeFormatOptions *options);
 
 void g_mime_object_encode (GMimeObject *object, GMimeEncodingConstraint constraint);
 
