@@ -295,23 +295,15 @@ decode_int (const char *in, size_t inlen)
 	inptr = in;
 	inend = in + inlen;
 	
-	if (*inptr == '-') {
-		sign = -1;
-		inptr++;
-	} else if (*inptr == '+') {
-		inptr++;
-	}
-	
 	while (inptr < inend) {
 		if (!(*inptr >= '0' && *inptr <= '9'))
 			return -1;
-		
+		if (val > (INT_MAX - (*inptr - '0')) / 10)
+			return -1;
 		val = (val * 10) + (*inptr - '0');
 		inptr++;
 	}
 	
-	val *= sign;
-	
 	return val;
 }
 
@@ -418,30 +410,36 @@ static gboolean
 get_time (const char *in, size_t inlen, int *hour, int *min, int *sec)
 {
 	register const char *inptr;
-	int *val, colons = 0;
+	int *val, max, colons = 0;
 	const char *inend;
 	
 	*hour = *min = *sec = 0;
 	
 	inend = in + inlen;
 	val = hour;
+	max = 23;
 	for (inptr = in; inptr < inend; inptr++) {
 		if (*inptr == ':') {
 			colons++;
 			switch (colons) {
 			case 1:
 				val = min;
+				max = 59;
 				break;
 			case 2:
 				val = sec;
+				max = 60;
 				break;
 			default:
 				return FALSE;
 			}
 		} else if (!(*inptr >= '0' && *inptr <= '9'))
 			return FALSE;
-		else
+		else {
 			*val = (*val * 10) + (*inptr - '0');
+			if (*val > max)
+				return FALSE;
+		}
 	}
 	
 	return TRUE;
@@ -466,7 +464,7 @@ get_tzone (date_token **token)
 			continue;
 		
 		if (len == 5 && (*inptr == '+' || *inptr == '-')) {
-			if ((value = decode_int (inptr, len)) == -1)
+			if ((value = decode_int (inptr + 1, len - 1)) == -1)
 				return NULL;
 			
 			memcpy (tzone, inptr, len);
