@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2014 Jeffrey Stedfast
+ *  Copyright (C) 2000-2017 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -23,13 +23,15 @@
 #include <config.h>
 #endif
 
+#include <stdlib.h>
 #include <time.h>
 
-#ifdef ENABLE_SMIME
+#ifdef ENABLE_CRYPTO
 #include <gpgme.h>
 #endif
 
 #include "gmime.h"
+#include "gmime-internal.h"
 
 #ifdef ENABLE_CRYPTOGRAPHY
 #include "gmime-pkcs7-context.h"
@@ -46,20 +48,7 @@
  * Initialization, shutdown, and version-check functions.
  **/
 
-extern gboolean _g_mime_enable_rfc2047_workarounds (void);
-extern gboolean _g_mime_use_only_user_charsets (void);
-
-extern void g_mime_iconv_utils_shutdown (void);
-extern void g_mime_iconv_utils_init (void);
-
-extern void _g_mime_iconv_cache_unlock (void);
-extern void _g_mime_iconv_cache_lock (void);
-extern void _g_mime_iconv_utils_unlock (void);
-extern void _g_mime_iconv_utils_lock (void);
-extern void _g_mime_charset_unlock (void);
-extern void _g_mime_charset_lock (void);
-extern void _g_mime_msgid_unlock (void);
-extern void _g_mime_msgid_lock (void);
+extern void g_mime_crypto_context_shutdown (void);
 
 GQuark gmime_gpgme_error_quark;
 GQuark gmime_error_quark;
@@ -70,13 +59,7 @@ const guint gmime_micro_version = GMIME_MICRO_VERSION;
 const guint gmime_interface_age = GMIME_INTERFACE_AGE;
 const guint gmime_binary_age = GMIME_BINARY_AGE;
 
-G_LOCK_DEFINE_STATIC (iconv_cache);
-G_LOCK_DEFINE_STATIC (iconv_utils);
-G_LOCK_DEFINE_STATIC (charset);
-G_LOCK_DEFINE_STATIC (msgid);
-
 static unsigned int initialized = 0;
-static guint32 enable = 0;
 
 
 /**
@@ -108,49 +91,41 @@ g_mime_check_version (guint major, guint minor, guint micro)
 
 /**
  * g_mime_init:
- * @flags: initialization flags
  *
  * Initializes GMime.
- *
- * Note: Calls g_mime_charset_map_init() and g_mime_iconv_init() as
- * well.
  **/
 void
-g_mime_init (guint32 flags)
+g_mime_init (void)
 {
+	initialized = MAX (initialized, 0);
+	
 	if (initialized++)
 		return;
 	
+	/* seed the random number generator (needed by boundary generator) */
+	srand ((unsigned int) time (NULL));
+	
 #if defined (HAVE_TIMEZONE) || defined (HAVE__TIMEZONE)
 	/* initialize timezone */
 	tzset ();
 #endif
 	
-	enable = flags;
-	
 #if !GLIB_CHECK_VERSION(2, 35, 1)
 	g_type_init ();
 #endif
 	
-#ifdef G_THREADS_ENABLED
-	g_mutex_init (&G_LOCK_NAME (iconv_cache));
-	g_mutex_init (&G_LOCK_NAME (iconv_utils));
-	g_mutex_init (&G_LOCK_NAME (charset));
-	g_mutex_init (&G_LOCK_NAME (msgid));
-#endif
-	
+	g_mime_format_options_init ();
+	g_mime_parser_options_init ();
 	g_mime_charset_map_init ();
-	g_mime_iconv_utils_init ();
-	g_mime_iconv_init ();
 	
-#ifdef ENABLE_SMIME
+#ifdef ENABLE_CRYPTO
 	/* gpgme_check_version() initializes GpgMe */
 	gpgme_check_version (NULL);
-#endif /* ENABLE_SMIME */
+#endif /* ENABLE_CRYPTO */
 	
 	gmime_gpgme_error_quark = g_quark_from_static_string ("gmime-gpgme");
 	gmime_error_quark = g_quark_from_static_string ("gmime");
-	
+
 	/* register our GObject types with the GType system */
 	g_mime_crypto_context_get_type ();
 	g_mime_decrypt_result_get_type ();
@@ -158,7 +133,7 @@ g_mime_init (guint32 flags)
 	g_mime_signature_list_get_type ();
 	g_mime_certificate_get_type ();
 	g_mime_signature_get_type ();
-#ifdef ENABLE_CRYPTOGRAPHY
+#ifdef ENABLE_CRYPTO
 	g_mime_gpg_context_get_type ();
 	g_mime_pkcs7_context_get_type ();
 #endif
@@ -167,13 +142,15 @@ g_mime_init (guint32 flags)
 	g_mime_filter_basic_get_type ();
 	g_mime_filter_best_get_type ();
 	g_mime_filter_charset_get_type ();
-	g_mime_filter_crlf_get_type ();
+	g_mime_filter_checksum_get_type ();
+	g_mime_filter_dos2unix_get_type ();
 	g_mime_filter_enriched_get_type ();
 	g_mime_filter_from_get_type ();
 	g_mime_filter_gzip_get_type ();
 	g_mime_filter_html_get_type ();
-	g_mime_filter_md5_get_type ();
+	g_mime_filter_smtp_data_get_type ();
 	g_mime_filter_strip_get_type ();
+	g_mime_filter_unix2dos_get_type ();
 	g_mime_filter_windows_get_type ();
 	g_mime_filter_yenc_get_type ();
 	
@@ -189,20 +166,31 @@ g_mime_init (guint32 flags)
 	g_mime_stream_null_get_type ();
 	g_mime_stream_pipe_get_type ();
 	
+	g_mime_format_options_get_type ();
+	g_mime_parser_options_get_type ();
+	
 	g_mime_parser_get_type ();
 	g_mime_message_get_type ();
 	g_mime_data_wrapper_get_type ();
-	g_mime_content_type_get_type ();
-	g_mime_content_disposition_get_type ();
 	
 	internet_address_get_type ();
 	internet_address_list_get_type ();
 	internet_address_group_get_type ();
 	internet_address_mailbox_get_type ();
 	
+	g_mime_param_get_type ();
+	g_mime_header_get_type ();
+	g_mime_param_list_get_type ();
+	g_mime_header_list_get_type ();
+	g_mime_content_type_get_type ();
+	g_mime_content_disposition_get_type ();
+	
 	/* register our default mime object types */
 	g_mime_object_type_registry_init ();
 	g_mime_object_register_type ("*", "*", g_mime_part_get_type ());
+	g_mime_object_register_type ("text", "*", g_mime_text_part_get_type ());
+	g_mime_object_register_type ("application", "x-pkcs7-mime", g_mime_application_pkcs7_mime_get_type ());
+	g_mime_object_register_type ("application", "pkcs7-mime", g_mime_application_pkcs7_mime_get_type ());
 	g_mime_object_register_type ("multipart", "*", g_mime_multipart_get_type ());
 	g_mime_object_register_type ("multipart", "encrypted", g_mime_multipart_encrypted_get_type ());
 	g_mime_object_register_type ("multipart", "signed", g_mime_multipart_signed_get_type ());
@@ -210,14 +198,25 @@ g_mime_init (guint32 flags)
 	g_mime_object_register_type ("message", "rfc2822", g_mime_message_part_get_type ());
 	g_mime_object_register_type ("message", "news", g_mime_message_part_get_type ());
 	g_mime_object_register_type ("message", "partial", g_mime_message_partial_get_type ());
+	
+	g_mime_crypto_context_register ("application/x-pgp-signature", g_mime_gpg_context_new);
+	g_mime_crypto_context_register ("application/pgp-signature", g_mime_gpg_context_new);
+	g_mime_crypto_context_register ("application/x-pgp-encrypted", g_mime_gpg_context_new);
+	g_mime_crypto_context_register ("application/pgp-encrypted", g_mime_gpg_context_new);
+	g_mime_crypto_context_register ("application/pgp-keys", g_mime_gpg_context_new);
+	
+	g_mime_crypto_context_register ("application/x-pkcs7-signature", g_mime_pkcs7_context_new);
+	g_mime_crypto_context_register ("application/pkcs7-signature", g_mime_pkcs7_context_new);
+	g_mime_crypto_context_register ("application/x-pkcs7-mime", g_mime_pkcs7_context_new);
+	g_mime_crypto_context_register ("application/pkcs7-mime", g_mime_pkcs7_context_new);
+	g_mime_crypto_context_register ("application/pkcs7-keys", g_mime_pkcs7_context_new);
 }
 
 
 /**
  * g_mime_shutdown:
  *
- * Frees internally allocated tables created in g_mime_init(). Also
- * calls g_mime_charset_map_shutdown() and g_mime_iconv_shutdown().
+ * Frees internally allocated tables created in g_mime_init().
  **/
 void
 g_mime_shutdown (void)
@@ -226,82 +225,8 @@ g_mime_shutdown (void)
 		return;
 	
 	g_mime_object_type_registry_shutdown ();
+	g_mime_crypto_context_shutdown ();
+	g_mime_format_options_shutdown ();
+	g_mime_parser_options_shutdown ();
 	g_mime_charset_map_shutdown ();
-	g_mime_iconv_utils_shutdown ();
-	g_mime_iconv_shutdown ();
-	
-#ifdef G_THREADS_ENABLED
-	if (glib_check_version (2, 37, 4) == NULL) {
-		/* The implementation of g_mutex_clear() prior
-		 * to glib 2.37.4 did not properly reset the
-		 * internal mutex pointer to NULL, so re-initializing
-		 * GMime would not properly re-initialize the mutexes.
-		 **/
-		g_mutex_clear (&G_LOCK_NAME (iconv_cache));
-		g_mutex_clear (&G_LOCK_NAME (iconv_utils));
-		g_mutex_clear (&G_LOCK_NAME (charset));
-		g_mutex_clear (&G_LOCK_NAME (msgid));
-	}
-#endif
-}
-
-
-gboolean
-_g_mime_enable_rfc2047_workarounds (void)
-{
-	return (enable & GMIME_ENABLE_RFC2047_WORKAROUNDS);
-}
-
-gboolean
-_g_mime_use_only_user_charsets (void)
-{
-	return (enable & GMIME_ENABLE_USE_ONLY_USER_CHARSETS);
-}
-
-void
-_g_mime_iconv_cache_unlock (void)
-{
-	return G_UNLOCK (iconv_cache);
-}
-
-void
-_g_mime_iconv_cache_lock (void)
-{
-	return G_LOCK (iconv_cache);
-}
-
-void
-_g_mime_iconv_utils_unlock (void)
-{
-	return G_UNLOCK (iconv_utils);
-}
-
-void
-_g_mime_iconv_utils_lock (void)
-{
-	return G_LOCK (iconv_utils);
-}
-
-void
-_g_mime_charset_unlock (void)
-{
-	return G_UNLOCK (charset);
-}
-
-void
-_g_mime_charset_lock (void)
-{
-	return G_LOCK (charset);
-}
-
-void
-_g_mime_msgid_unlock (void)
-{
-	return G_UNLOCK (msgid);
-}
-
-void
-_g_mime_msgid_lock (void)
-{
-	return G_LOCK (msgid);
 }
