@@ -336,6 +336,9 @@ static struct {
 	{ "17-6-2008 17:10:08",
 	  "Tue, 17 Jun 2008 17:10:08 +0000",
 	  1213722608, 0 },
+	{ "Sat, 28 Oct 2017 19:41:29 -0001",
+	  "Sat, 28 Oct 2017 19:41:29 -0001",
+	  1509219749, -1 },
 	{ "nonsense",
 	  "Thu, 01 Jan 1970 00:00:00 +0000",
 	  0, 0 }
