@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2014 Jeffrey Stedfast
+ *  Copyright (C) 2000-2017 Jeffrey Stedfast
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -34,14 +34,12 @@
 
 #include "testsuite.h"
 
-#define GPG_PATH "gpg"
-
 extern int verbose;
 
 #define v(x) if (verbose > 3) x
 
 static gboolean
-request_passwd (GMimeCryptoContext *ctx, const char *user_id, const char *prompt_ctx, gboolean reprompt, GMimeStream *response, GError **err)
+request_passwd (GMimeCryptoContext *ctx, const char *user_id, const char *prompt, gboolean reprompt, GMimeStream *response, GError **err)
 {
 	g_mime_stream_write_string (response, "no.secret\n");
 	
@@ -51,31 +49,29 @@ request_passwd (GMimeCryptoContext *ctx, const char *user_id, const char *prompt
 static GMimeSignatureStatus
 get_sig_status (GMimeSignatureList *signatures)
 {
-	GMimeSignatureStatus status = GMIME_SIGNATURE_STATUS_GOOD;
+	GMimeSignatureStatus status = 0;
 	GMimeSignature *sig;
 	int i;
 	
 	if (!signatures || signatures->array->len == 0)
-		return GMIME_SIGNATURE_STATUS_ERROR;
+		return GMIME_SIGNATURE_STATUS_RED;
 	
 	for (i = 0; i < g_mime_signature_list_length (signatures); i++) {
 		sig = g_mime_signature_list_get_signature (signatures, i);
-		status = MAX (status, sig->status);
+		status |= g_mime_signature_get_status (sig);
 	}
 	
 	return status;
 }
 
 static void
-test_sign (GMimeCryptoContext *ctx, GMimeStream *cleartext, GMimeStream *ciphertext)
+test_sign (GMimeCryptoContext *ctx, gboolean detached, GMimeStream *cleartext, GMimeStream *ciphertext)
 {
 	GError *err = NULL;
 	Exception *ex;
 	int rv;
 	
-	rv = g_mime_crypto_context_sign (ctx, "no.user@no.domain",
-					 GMIME_DIGEST_ALGO_SHA256,
-					 cleartext, ciphertext, &err);
+	rv = g_mime_crypto_context_sign (ctx, detached, "no.user@no.domain", cleartext, ciphertext, &err);
 	
 	if (rv == -1 || err != NULL) {
 		ex = exception_new ("%s", err->message);
@@ -90,14 +86,14 @@ test_sign (GMimeCryptoContext *ctx, GMimeStream *cleartext, GMimeStream *ciphert
 }
 
 static void
-test_verify (GMimeCryptoContext *ctx, GMimeStream *cleartext, GMimeStream *ciphertext)
+test_verify_detached (GMimeCryptoContext *ctx, GMimeStream *cleartext, GMimeStream *ciphertext)
 {
 	GMimeSignatureList *signatures;
+	GMimeSignatureStatus status;
 	GError *err = NULL;
 	Exception *ex;
 	
-	signatures = g_mime_crypto_context_verify (ctx, GMIME_DIGEST_ALGO_DEFAULT,
-						   cleartext, ciphertext, &err);
+	signatures = g_mime_crypto_context_verify (ctx, 0, cleartext, ciphertext, NULL, &err);
 	
 	if (signatures == NULL) {
 		ex = exception_new ("%s", err->message);
@@ -105,7 +101,9 @@ test_verify (GMimeCryptoContext *ctx, GMimeStream *cleartext, GMimeStream *ciphe
 		throw (ex);
 	}
 	
-	if (get_sig_status (signatures) != GMIME_SIGNATURE_STATUS_GOOD) {
+	status = get_sig_status (signatures);
+	
+	if ((status & GMIME_SIGNATURE_STATUS_RED) != 0) {
 		g_object_unref (signatures);
 		throw (exception_new ("signature BAD"));
 	}
@@ -113,6 +111,50 @@ test_verify (GMimeCryptoContext *ctx, GMimeStream *cleartext, GMimeStream *ciphe
 	g_object_unref (signatures);
 }
 
+static void
+test_verify (GMimeCryptoContext *ctx, GMimeStream *cleartext, GMimeStream *ciphertext)
+{
+	GMimeSignatureList *signatures;
+	GMimeSignatureStatus status;
+	Exception *ex = NULL;
+	GMimeStream *stream;
+	GError *err = NULL;
+	GByteArray *buf[2];
+	
+	stream = g_mime_stream_mem_new ();
+	
+	signatures = g_mime_crypto_context_verify (ctx, 0, ciphertext, NULL, stream, &err);
+	
+	if (signatures == NULL) {
+		ex = exception_new ("%s", err->message);
+		g_object_unref (stream);
+		g_error_free (err);
+		throw (ex);
+	}
+	
+	status = get_sig_status (signatures);
+	
+	if ((status & GMIME_SIGNATURE_STATUS_RED) != 0) {
+		g_object_unref (signatures);
+		g_object_unref (stream);
+		
+		throw (exception_new ("signature BAD"));
+	}
+	
+	g_object_unref (signatures);
+	
+	buf[0] = GMIME_STREAM_MEM (cleartext)->buffer;
+	buf[1] = GMIME_STREAM_MEM (stream)->buffer;
+	
+	if (buf[0]->len != buf[1]->len || memcmp (buf[0]->data, buf[1]->data, buf[0]->len) != 0)
+		ex = exception_new ("extracted data does not match original cleartext");
+	
+	g_object_unref (stream);
+	
+	if (ex != NULL)
+		throw (ex);
+}
+
 static void
 test_encrypt (GMimeCryptoContext *ctx, gboolean sign, GMimeStream *cleartext, GMimeStream *ciphertext)
 {
@@ -123,8 +165,10 @@ test_encrypt (GMimeCryptoContext *ctx, gboolean sign, GMimeStream *cleartext, GM
 	recipients = g_ptr_array_new ();
 	g_ptr_array_add (recipients, "no.user@no.domain");
 	
-	g_mime_crypto_context_encrypt (ctx, sign, "no.user@no.domain", GMIME_DIGEST_ALGO_SHA256,
-				       recipients, cleartext, ciphertext, &err);
+	g_mime_crypto_context_encrypt (ctx, sign, "no.user@no.domain",
+				       GMIME_ENCRYPT_ALWAYS_TRUST,
+				       recipients, cleartext, ciphertext,
+				       &err);
 	
 	g_ptr_array_free (recipients, TRUE);
 	
@@ -142,6 +186,8 @@ test_encrypt (GMimeCryptoContext *ctx, gboolean sign, GMimeStream *cleartext, GM
 static void
 test_decrypt (GMimeCryptoContext *ctx, gboolean sign, GMimeStream *cleartext, GMimeStream *ciphertext)
 {
+	GMimeSignatureList *signatures;
+	GMimeSignatureStatus status;
 	GMimeDecryptResult *result;
 	Exception *ex = NULL;
 	GMimeStream *stream;
@@ -150,18 +196,26 @@ test_decrypt (GMimeCryptoContext *ctx, gboolean sign, GMimeStream *cleartext, GM
 	
 	stream = g_mime_stream_mem_new ();
 	
-	if (!(result = g_mime_crypto_context_decrypt (ctx, ciphertext, stream, &err))) {
+	if (!(result = g_mime_crypto_context_decrypt (ctx, 0, NULL, ciphertext, stream, &err))) {
 		g_object_unref (stream);
 		ex = exception_new ("%s", err->message);
 		g_error_free (err);
 		throw (ex);
 	}
 	
+	signatures = g_mime_decrypt_result_get_signatures (result);
+	
 	if (sign) {
-		if (!result->signatures || get_sig_status (result->signatures) != GMIME_SIGNATURE_STATUS_GOOD)
-			ex = exception_new ("expected GOOD signature");
+		if (signatures != NULL) {
+			status = get_sig_status (signatures);
+			
+			if ((status & GMIME_SIGNATURE_STATUS_RED) != 0)
+				ex = exception_new ("expected GOOD signature");
+		} else {
+			ex = exception_new ("Failed to get signatures");
+		}
 	} else {
-		if (result->signatures != NULL)
+		if (signatures != NULL)
 			ex = exception_new ("unexpected signature");
 	}
 	
@@ -177,8 +231,8 @@ test_decrypt (GMimeCryptoContext *ctx, gboolean sign, GMimeStream *cleartext, GM
 		throw (ex);
 	}
 	
-	buf[0] = GMIME_STREAM_MEM (cleartext)->buffer;
-	buf[1] = GMIME_STREAM_MEM (stream)->buffer;
+	buf[0] = g_mime_stream_mem_get_byte_array ((GMimeStreamMem *) cleartext);
+	buf[1] = g_mime_stream_mem_get_byte_array ((GMimeStreamMem *) stream);
 	
 	if (buf[0]->len != buf[1]->len || memcmp (buf[0]->data, buf[1]->data, buf[0]->len) != 0)
 		ex = exception_new ("decrypted data does not match original cleartext");
@@ -197,8 +251,8 @@ test_export (GMimeCryptoContext *ctx, const char *path)
 	const char *inbuf, *outbuf;
 	size_t inlen, outlen;
 	Exception *ex = NULL;
+	const char *keys[2];
 	GError *err = NULL;
-	GPtrArray *keys;
 	int fd;
 	
 	if ((fd = open (path, O_RDONLY, 0)) == -1)
@@ -210,13 +264,13 @@ test_export (GMimeCryptoContext *ctx, const char *path)
 	g_mime_stream_reset (istream);
 	g_object_unref (ostream);
 	
-	keys = g_ptr_array_new ();
-	g_ptr_array_add (keys, "no.user@no.domain");
+	keys[0] = "no.user@no.domain";
+	keys[1] = NULL;
 	
 	ostream = g_mime_stream_mem_new ();
 	
 	g_mime_crypto_context_export_keys (ctx, keys, ostream, &err);
-	g_ptr_array_free (keys, TRUE);
+	
 	if (err != NULL) {
 		ex = exception_new ("%s", err->message);
 		g_object_unref (istream);
@@ -278,21 +332,28 @@ import_key (GMimeCryptoContext *ctx, const char *path)
 
 int main (int argc, char **argv)
 {
+#ifdef ENABLE_CRYPTO
 	const char *datadir = "data/pgp";
 	GMimeStream *istream, *ostream;
 	GMimeCryptoContext *ctx;
 	const char *what;
+	char *gpg, *key;
 	struct stat st;
-	char *key;
 	int i;
 	
-	g_mime_init (0);
+	g_mime_init ();
 	
 	testsuite_init (argc, argv);
 	
-	if (testsuite_setup_gpghome (GPG_PATH) != 0)
+	if (!(gpg = g_find_program_in_path ("gpg2")))
+		if (!(gpg = g_find_program_in_path ("gpg")))
+			return EXIT_FAILURE;
+	
+	if (testsuite_setup_gpghome (gpg) != 0)
 		return EXIT_FAILURE;
 	
+	g_free (gpg);
+	
 	for (i = 1; i < argc; i++) {
 		if (argv[i][0] != '-') {
 			datadir = argv[i];
@@ -305,8 +366,8 @@ int main (int argc, char **argv)
 	
 	testsuite_start ("GnuPG crypto context");
 	
-	ctx = g_mime_gpg_context_new (request_passwd, GPG_PATH);
-	g_mime_gpg_context_set_always_trust ((GMimeGpgContext *) ctx, TRUE);
+	ctx = g_mime_gpg_context_new ();
+	g_mime_crypto_context_set_request_password (ctx, request_passwd);
 	
 	testsuite_check ("GMimeGpgContext::import");
 	try {
@@ -344,7 +405,7 @@ int main (int argc, char **argv)
 	what = "GMimeGpgContext::sign";
 	testsuite_check (what);
 	try {
-		test_sign (ctx, istream, ostream);
+		test_sign (ctx, FALSE, istream, ostream);
 		testsuite_check_passed ();
 		
 		what = "GMimeGpgContext::verify";
@@ -361,6 +422,26 @@ int main (int argc, char **argv)
 	g_mime_stream_reset (istream);
 	ostream = g_mime_stream_mem_new ();
 	
+	what = "GMimeGpgContext::sign (detached)";
+	testsuite_check (what);
+	try {
+		test_sign (ctx, TRUE, istream, ostream);
+		testsuite_check_passed ();
+		
+		what = "GMimeGpgContext::verify (detached)";
+		testsuite_check (what);
+		g_mime_stream_reset (istream);
+		g_mime_stream_reset (ostream);
+		test_verify_detached (ctx, istream, ostream);
+		testsuite_check_passed ();
+	} catch (ex) {
+		testsuite_check_failed ("%s failed: %s", what, ex->message);
+	} finally;
+	
+	g_object_unref (ostream);
+	g_mime_stream_reset (istream);
+	ostream = g_mime_stream_mem_new ();
+	
 	what = "GMimeGpgContext::encrypt";
 	testsuite_check (what);
 	try {
@@ -409,4 +490,8 @@ int main (int argc, char **argv)
 		return EXIT_FAILURE;
 	
 	return testsuite_exit ();
+#else
+	fprintf (stderr, "PGP support not enabled in this build.\n");
+	return EXIT_SUCCESS;
+#endif
 }
